/* SPDX-License-Identifier: GPL-2.0 */
/*
 *  Kylin security modules
 *
 *  This file contains the Kylin security implementations.
 *
 *  Authors:  penghuaihe <penghuaihe@kylinos.cn>
 *
 *  Copyright (C) 2024, KylinSoft Corporation. All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2,
 *  as published by the Free Software Foundation.
 */

#ifndef _KSAF_FRAMEWORK_H_
#define _KSAF_FRAMEWORK_H_

#include <linux/errno.h>
#include <linux/list.h>
#include <linux/spinlock.h>
#include <linux/lsm_hooks.h>
#include <linux/kernel.h>
#include <linux/cred.h>
#include <linux/binfmts.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/usb.h>
#include <linux/usb/hcd.h>

#define KSAF_SCENE_MAX			32

/* Security label count ksaf support. Exclude selinux, smack and common */
#define KSAF_INODE_XATTRS_MAX 5

struct usb_interface;
struct device;

typedef union {
	struct {
		uint64_t pkg_id;
		uint64_t file_id;
		uint64_t path_id;
	};
	uint64_t id;
} KSAF_SID;

struct ksaf_common_inode_security {
	struct inode *inode;	/* back pointer to inode object */

	KSAF_SID exec_sid;
	uint32_t t_id;
	KSAF_SID oid;
	uint32_t inherit;
	KSAF_SID creator;

	uint32_t oclass;
	uint32_t i_id;
	uint32_t m_id;

	unsigned char initialized;
	struct timespec64 sec_ctime;
	spinlock_t lock;	/* used to inode security data set */
};

struct ksaf_common_file_security {
	spinlock_t lock;	/* used to file security data set */
};

struct ksaf_common_task_security {
	KSAF_SID sid;
	KSAF_SID orig_sid;

	uint32_t t_id;
	uint32_t i_id;
	uint32_t m_id;

	struct timespec64 sec_ctime;
};

struct ksaf_common_sb_security {
	struct super_block *sb;		/* back pointer to sb object */
	unsigned short flags;		/* which mount options were specified */

	spinlock_t lock;	/* used to sb security data set */
};

/*
 * pre-alloc all label memory. Notice that member labels just is a pointer
 * here. Security modules which want to use a private label must call
 * ksaf_label_slot_request() to request a slot in array labels[], and register
 * their inode_alloc/free_security hooks to init/destroy the polinter.
 */
struct ksaf_framework_inode_security {
	void *labels[KSAF_INODE_XATTRS_MAX];

	/*
	 * As common label need initialize the 0 task, common label should be
	 * located in kernel.
	 */
	struct ksaf_common_inode_security ksaf_common;
};

struct ksaf_framework_file_security {
	void *labels[KSAF_INODE_XATTRS_MAX];

	struct ksaf_common_file_security ksaf_common;
};

struct ksaf_framework_task_security {
	void *labels[KSAF_INODE_XATTRS_MAX];
	struct ksaf_common_task_security ksaf_common;
};

struct ksaf_framework_sb_security {
	void *labels[KSAF_INODE_XATTRS_MAX];
	struct ksaf_common_sb_security ksaf_common;
};

extern struct lsm_blob_sizes ksaf_blob_sizes;
static inline struct ksaf_framework_inode_security *ksaf_isec(const struct inode *inode)
{
	return inode->i_security + ksaf_blob_sizes.lbs_inode;
}

static inline struct ksaf_common_inode_security
	*ksaf_common_isec(const struct inode *inode)
{
	if (unlikely(!inode->i_security))
		return NULL;
	return &(ksaf_isec(inode)->ksaf_common);
}

static inline struct ksaf_framework_file_security *ksaf_fsec(const struct file *file)
{
	return file->f_security + ksaf_blob_sizes.lbs_file;
}

static inline struct ksaf_common_file_security
	*ksaf_common_fsec(const struct file *file)
{
	if (unlikely(!file->f_security))
		return NULL;
	return &(ksaf_fsec(file)->ksaf_common);
}

static inline struct ksaf_framework_task_security *ksaf_cred(const struct cred *cred)
{
	return cred->security + ksaf_blob_sizes.lbs_cred;
}

static inline struct ksaf_common_task_security
	*ksaf_common_cred(const struct cred *cred)
{
	if (unlikely(!cred->security))
		return NULL;
	return &(ksaf_cred(cred)->ksaf_common);
}

static inline struct ksaf_framework_task_security *ksaf_tsec(const struct task_struct *task)
{
	return task->security + ksaf_blob_sizes.lbs_task;
}

static inline struct ksaf_framework_sb_security *ksaf_sbsec(const struct super_block *sb)
{
	return sb->s_security + ksaf_blob_sizes.lbs_superblock;
}

static inline struct ksaf_common_task_security *ksaf_common_tsec(const struct task_struct *task)
{
	if (unlikely(!task->security))
		return NULL;
	return &(ksaf_tsec(task)->ksaf_common);
}

static inline void *ksaf_extmod_isec(const struct inode *inode, int slot)
{
	if (unlikely(!inode->i_security))
		return NULL;
	return ksaf_isec(inode)->labels[slot];
}

static inline void
ksaf_extmod_inode_set_blob(const struct inode *inode, void *blob, int slot)
{
	ksaf_isec(inode)->labels[slot] = blob;
}

static inline void
ksaf_extmod_inode_free_blob(const struct inode *inode, void *blob, int slot)
{
	kfree(blob);
	ksaf_isec(inode)->labels[slot] = NULL;
}

static inline void *ksaf_extmod_tsec(const struct task_struct *task, int slot)
{
	if (unlikely(!task->security))
		return NULL;
	return ksaf_tsec(task)->labels[slot];
}

static inline void
ksaf_extmod_task_set_blob(const struct task_struct *task, void *blob, int slot)
{
	ksaf_tsec(task)->labels[slot] = blob;
}

static inline void
ksaf_extmod_task_free_blob(const struct task_struct *task, void *blob, int slot)
{
	kfree(blob);
	ksaf_tsec(task)->labels[slot] = NULL;
}

static inline void *ksaf_extmod_sbsec(const struct super_block *sb, int slot)
{
	if (unlikely(!sb->s_security))
		return NULL;
	return ksaf_sbsec(sb)->labels[slot];
}

static inline void
ksaf_extmod_sb_set_blob(const struct super_block *sb, void *blob, int slot)
{
	ksaf_sbsec(sb)->labels[slot] = blob;
}

static inline void
ksaf_extmod_sb_free_blob(const struct super_block *sb, void *blob, int slot)
{
	kfree(blob);
	ksaf_sbsec(sb)->labels[slot] = NULL;
}

struct task_struct *ksaf_find_task_by_vpid(pid_t vnr);
pid_t ksaf_task_ppid_nr(const struct task_struct *tsk);
pid_t ksaf_task_pid_nr(struct task_struct *tsk);
pid_t ksaf_task_pgrp_nr(struct task_struct *tsk);

int ksaf_label_slot_request(void);
char *ksaf_get_filepath_by_dentry(struct dentry *dentry);
char *ksaf_get_filepath_by_inode(struct inode *inode);
int ksaf_run_cmd(const char *cmd, int wait);
int ksaf_get_task_tpgid(struct task_struct *task);

unsigned long ksaf_module_kallsyms_lookup_name(const char *name);

int ksaf_kernel_read(struct file *file, loff_t offset,
		     void *addr, unsigned long count);

unsigned int ksaf_get_task_inum(struct task_struct *task);

int ksaf_vfs_setxattr_noperm(struct mnt_idmap *idmap,
			     struct dentry *dentry, const char *name,
			     const void *value, size_t size, int flags);

int ksaf_for_each_usb_dev(int (*fn)(struct device *, void *));
void ksaf_usb_forced_unbind_intf(struct usb_interface *intf);
void ksaf_usb_rebind_intf(struct usb_interface *intf);
ssize_t ksaf_read_task_cmdline(struct task_struct *tsk, char *buf, size_t count);
char *ksaf_get_filepath_by_task(struct task_struct *task);
int ksaf_usb_hcd_find_raw_port_number(struct usb_hcd *hcd, int port1);

#endif
