/* SPDX-License-Identifier: (GPL-2.0 WITH Linux-syscall-note) */
/*
 * Header file for the kalert interface.
 *
 * Copyright (C) 2024 KylinSoft Corporation.
 * Authors: Huiwen He <hehuiwen@kylinos.cn>
 */

#ifndef _UAPI_LINUX_KALERT_H
#define _UAPI_LINUX_KALERT_H
#include <linux/types.h>

#define NETLINK_KALERT 30

/* The netlink messages for the alert system is divided into blocks:
 * 1000 - 1099 are for commanding the alert subsystem
 * 1100 - 1199 are for kernel space alert subsystem to notify user space
 *
 * Messages from 1000-1099 are bi-directional，
 * 1100-1199 are kernel --> user space communication.
 */
#define KALERT_CMD_SET_CHNL 1000	/* set control param for kalert to notify */
#define KALERT_CMD_GET_VERSION 1001	/* get kalert protocol version */
#define KALERT_CMD_SET_LEVEL 1002	/* set alert level to filter messages */
#define KALERT_CMD_GET_STATUS 1003	/* get kalert notify send statistics */
#define KALERT_CMD_SUBSCRIBE 1004	/* subscribe notify */
#define KALERT_CMD_UNSUBSCRIBE 1005	/* unscribe notify */

#define KALERT_NOTIFY_TYPE_FIRST 1100	/* first notify massage type */
#define KALERT_NOTIFY_TYPE_LAST  1199	/* last notify message type */

/* kalert channel param control mask*/
#define KALERT_CHNL_ENABLE 0x0001
#define KALERT_CHNL_PORTID 0x0002
#define KALERT_CHNL_LEVEL 0x0004
#define KALERT_CHNL_BACKLOG_LIMIT 0x0008
#define KALERT_CHNL_LOST 0x0010

/* max messages len */
#define KALERT_NOTIFY_INFO_MAX 256

/* type used to classify notification */
enum kalert_notify_type {
	KALERT_NOTIFY_ALL = 0,
	KALERT_NOTIFY_GEN,
	KALERT_NOTIFY_MEM,
	KALERT_NOTIFY_IO,
	KALERT_NOTIFY_FS,
	KALERT_NOTIFY_TYPE_MAX,
};

/* level used to filter notification */
enum kalert_notify_level {
	KALERT_LEVEL_ALL = 0,
	KALERT_LEVEL_DEBUG,
	KALERT_LEVEL_WARN,
	KALERT_LEVEL_ERROR,
	KALERT_LEVEL_FATAL,
	KALERT_LEVEL_MAX,
};

/* specific event of gen type to subscribe */
enum kalert_gen_event {
	KALERT_GEN_ALL = 0,
	KALERT_GEN_SOFTLOCKUP,
	KALERT_GEN_RCUSTALL,
	KALERT_GEN_EVENT_MAX
};

/* specific event of mem type to subscribe */
enum kalert_mem_event {
	KALERT_MEM_ALL = 0,
	KALERT_MEM_ALLOCFAIL,
	KALERT_MEM_OOM,
	KALERT_MEM_BAD_STATE,
	KALERT_MEM_LEAK,
	KALERT_MEM_EVENT_MAX
};

/* specific event of IO type to subscribee */
enum kalert_io_event {
	KALERT_IO_ALL = 0,
	KALERT_IO_EVENT_MAX
};

/* specific event to of fs type to subscribe */
enum kalert_fs_event {
	KALERT_FS_ALL = 0,
	KALERT_FS_EVENT_MAX
};

/* message content to filled by kernel modules */
struct kalert_notify_msg {
	__u32 type;		/* notification type */
	__u32 level;		/* notification level */
	__u32 event;		/* notification event more specific */
	char info[KALERT_NOTIFY_INFO_MAX];	/* specific messages content */
};

/* filled by kalertd to set channel parameters */
struct kalert_ctl_msg {
	__u32 mask;		/* mask for kalertd to specify items to be modified */
	__u32 enable;		/* kalert framework enable or disable */
	__u32 portid;		/* portid for kernel to notify kalertd */
	__u32 level;		/* level to filter alert notification */
	__u32 backlog_limit;	/* max backlog queue length to adjust Qos */
	__u32 lost;		/* used to reset packet loss rate statistics */
};

/* status of current channel */
struct kalert_status {
	__u32 enabled;		/* whether kalert framework enable or not */
	__u32 lost;		/* packet loss count due to backlog queue Qos */
	__u32 level;		/* current kalertd subscribe alert level */
	__u32 portid;		/* current kalertd's portid */
	__u32 backlog_notify_depth;	/* current backlog queue depth */
	__u32 backlog_notify_limit;	/* max backlog queue depth limit */
};

/* request filled by subscriber */
struct kalert_subscribe_request {
	__u32 portid;		/* user space must filled with it's pid */
	unsigned long type_mask;	/* listen msg type */
	__u32 level;		/* listen msg >= level */
	__u32 event[KALERT_NOTIFY_TYPE_MAX];	/* listen specific event */
};

#endif
