#!/bin/bash
#
# SPDX-License-Identifier: BSD-2-Clause
#
# Copyright (c) 2020-2021 Senscomm Semiconductor Co., Ltd.
#

usage()
{
	prog=`basename $0`
	cat << USAGE
Usage: $prog <directory>
USAGE
}

if [[ $# < 1 ]]; then
	usage
	exit 1
fi

if [ -z $1 ] || [ ! -d $1 ]; then
	usage
	exit 1
fi

mac=`ls $1/mac*.h`
phy=`ls $1/phy*.h`


#
# Heading
#
cat << HEADER
/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 *
 */
#ifndef __SCM80211_REGS_H__
#define __SCM80211_REGS_H__

HEADER


echo "#include \"$mac\""
echo "#include \"$phy\""
if [ -e $1/errata.h ]; then
	echo "#include $1/errata.h"
fi

#
# MAC registers
#

cat << REG

/*
 * MAC register map
 */

#define MAC_IMEM_OFFSET			0x0000
#define MAC_DMEM_OFFSET			0x4000
#define MAC_SHM_OFFSET			0x4760
#define MAC_FIFO_OFFSET			0x6000
#define MAC_MISC_OFFSET			0x7000
#define MAC_REG_OFFSET			0x8000
#define MAC_SYS_OFFSET			0x9000
#define MAC_TIM_OFFSET			0x9100
#define MAC_TXE_OFFSET			0x9300

/*
 * MAC shared memory region
 */

#define MAC_SHM_TXDESC(x)		(MAC_SHM_OFFSET + ((x) * 0x10))
#define TXDESC_MDESC_ADDR_LO(x)		(MAC_SHM_TXDESC(x) + 0x0)
#define TXDESC_MDESC_ADDR_HI(x)		(MAC_SHM_TXDESC(x) + 0x4)
#define TXDESC_NUM_MPDU_RESULT(x)	(MAC_SHM_TXDESC(x) + 0x8)
#define		TXDESC_NUM_MPDU		GENMASK(5, 0)
#define TXDESC_TX_HTC(x)		(MAC_SHM_TXDESC(x) + 0xc)

#define MAC_SHM_RXDESC(x)		(MAC_SHM_OFFSET + 0xa0 + ((x) * 0x10))
#define RXDESC_BUF_ADDR_LO(x)		(MAC_SHM_RXDESC(x) + 0x0)
#define RXDESC_BUF_ADDR_HI(x)		(MAC_SHM_RXDESC(x) + 0x4)
#define RXDESC_BUF_SIZE(x)		(MAC_SHM_RXDESC(x) + 0x8)
#define MAC_MISC_NULL_HEADER		(MAC_MISC_OFFSET + 0x13C)
#define TIM_IFS_CFG1			(MAC_TIM_OFFSET + 0x14)
#define TXE_TIM_INFO0			(MAC_TXE_OFFSET + 0x10)

/*
 * MAC register definitions
 * #define MAC_REG_INTR_ENABLE 	(MAC_REG_OFFSET + REG_INTR_ENABLE)
 */

REG

grep REG_ $mac | grep define |\
cut -d ' ' -f1,2 |\
sed 's/\(REG_.*\)/MAC_\1\t\t(MAC_REG_OFFSET + \1)/g'  |\
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'

cat << VIF

/*
 * MAC VIF register macros
 */

#define REGS_VIF_STRIDE	(REG_VIF1_CFG0 - REG_VIF0_CFG0)
#define VIF(n)		((n) * REGS_VIF_STRIDE)

VIF


grep VIF1 $mac | grep define | grep REG_ |\
cut -d ' ' -f1,2 | \
sed 's/\(REG_VIF\)1\(_.*\)/MAC_\1\2(v)\t\t(MAC_\10\2 + VIF(v))/g' | \
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'

#
#
cat << MASK

/*
 * Register field definitions for <linux/bitfield.h>
 * #define VIF_CFG0_EN		VIF0_CFG0_EN_MASK
 */

MASK

# (INTR|DEV)_XXX_MASK
grep -e "_MASK[^_]" $mac			|\
grep -v "[^_]\(VIF[01]\|TX_QUEN\|DBG\)_"	|\
sed -e 's/[[:space:]]\+/ /g'			|\
cut -d ' ' -f1,2 				|\
sed 's/\([^[:space:]]*\)_MASK/\1\t\t\1_MASK/g' 	|\
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'

echo ""
echo "/* VIFXXX_MASK */"
grep -e "[^_]VIF0.*_MASK[^_]" $mac	|\
sed -e 's/[[:space:]]\+/ /g'	|\
cut -d ' ' -f1,2 		|\
sed 's/\(VIF\)0\(.*\)_MASK/\1\2\t\t\10\2_MASK/g' |\
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'

echo ""
echo "/* TX_QUEN_XXX_MASK */"
grep -e "TX_QUEN_.*0.*_MASK[^_]" $mac		|\
sed -e 's/[[:space:]]\+/ /g'			|\
cut -d ' ' -f1,2 				|\
sed 's/\(TX_QUEN_.*\)0\(.*\)_MASK/\1\2\t\t\10\2_MASK/g' 	|\
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'

cat << TX
/* Add unused define*/
#ifndef VIF_CFG0_SLOT_TIME
#define VIF_CFG0_SLOT_TIME			0xFF0000
#endif

#ifndef DEV_CFG_SLOT_TIME
#define DEV_CFG_SLOT_TIME			0x01
#endif

#ifndef RX_BUF_CFG_DMA_EN
#define RX_BUF_CFG_DMA_EN			0x01
#endif

/* TX registers */
#define REG_TX_STRIDE				(REG_TX_QUEN_CMD1 - REG_TX_QUEN_CMD0)
#define TXQ(n)					((n) * REG_TX_STRIDE)

#define MAC_REG_TX_QUEN_CMD(n)			(TXQ(n) + MAC_REG_TX_QUEN_CMD0)
#define MAC_REG_TX_QUEN_TXOP(n)			(TXQ(n) + MAC_REG_TX_QUEN_TXOP0)
#define MAC_REG_TX_QUEN_STATE(n)		(TXQ(n) + MAC_REG_TX_QUEN_STATE0)
#define MAC_REG_TX_QUEN_TIMER(n)		(TXQ(n) + MAC_REG_TX_QUEN_TIMER0)
#define MAC_REG_VIF_QNULL_CTRL_AC(v, ac)	(MAC_REG_VIF_QNULL_CTRL_AC0(v) + (ac) * 4)

/* Bit fields */
#define VIF_QNULL_CTRL_AC_SEQUNECE_CTRL        	VIF0_QNULL_CTRL_AC0_SEQUNECE_CTRL_MASK
#define VIF_QNULL_CTRL_AC_QOS_CTRL            	VIF0_QNULL_CTRL_AC0_QOS_CTRL_MASK

#define TX_QUEN_CMD_EDCA			BIT(0)
#define TX_QUEN_CMD_AID_MATCHED_QUEUE_EN	BIT(1)
#define TX_QUEN_CMD_ASSO_UORA_QUEUE_EN		BIT(2)
#define TX_QUEN_CMD_UNASSO_UORA_QUEUE_EN	BIT(3)
#define TX_QUEN_CMD_HE_TB_RESP_MASK		(TX_QUEN_CMD_CMD_EDCA & ~TX_QUEN_CMD_EDCA)

TX

cat << DBG

/* Debug registers */
#define REG_DBG_RX_PPDU_CNT(n)		(REG_DBG_RX_PPDU_CNT0 + 0x04 * (n))

DBG


cat << MODEM

/**
 * PHY registers supplimentary definitions
 *
 * @PHY_BASE(i): Base address of PHY i
 * @PHY_MODEM(reg): Modem register offset
 * @PHY_RFIF(reg): RF interface register offset
 */
#define PHY_MODEM_OFFSET	0x0
#define PHY_RFI_OFFSET          0x8000

#define PHY_BASE(i)		((i) * 0x4000)
#define PHY_MODEM(reg)		(reg)
#define PHY_RFIF(reg)		(PHY_RFI_OFFSET + (reg))
MODEM

#
#

cat << PHY_VIF

/*
 * PHY VIF (PHY context) banked registers
 *
 * To support fast VIF (or channel) switching, the following registers are
 * banked. Bank selection is done by setting FW_SEL_VIF (0: bank 0, 1: bank 1)
 *
 * #define FW_OPBW_PRICH(v)	(FW_OPBW_PRICH_VINTF0 + PHY_CTX(v))
 */

#define PHY_VINTF_STRIDE        (FW_OPBW_PRICH_VINTF1 - FW_OPBW_PRICH_VINTF0)
#define PHY_CTX(v)		((v) * PHY_VINTF_STRIDE)

PHY_VIF

grep -v "MASK\|SHIFT" $phy | grep VINTF0	|\
cut -d ' ' -f1,2 				|\
sed 's/ \(.*\)_VINTF0/ \1_VINTF(v)\t\t(\1_VINTF0 + PHY_CTX(v))/g' |\
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'

unused() {
echo ""
echo "/* PHY register bitfield short name */"
echo ""

grep VINTF0_MASK $phy	|\
sed -e 's/[[:space:]]\+/ /g'	|\
cut -d ' ' -f1,2 		|\
sed 's/ \(.*\)_VINTF0_MASK/ \1\t\t\1_VINTF0_MASK/g'	|\
awk '{printf "%s %-45s\t%s\n", $1, $2, substr($0, index($0, $3))}'
}

cat << RFADD
/**
 * RF physical address
 */

#define RF_BASE_START		0x0

RFADD

cat << RFI

/**
 * RF interface
 */

/* RFI_CTRLSIG_CFG_XX fields */
#define RFI_CTRLSIG_CFG(x)	PHY_RFIF(FW_RFI_CTRLSIG_CFG_00 + (x * 4))

#define RFI_CTRLSIG_CFG_EN	BIT(21)
#define RFI_CTRLSIG_CFG_POL	BIT(20)
#define RFI_CTRLSIG_CFG_SSEL	GENMASK(19, 18)
#define RFI_CTRLSIG_CFG_ESEL	GENMASK(17, 16)
#define RFI_CTRLSIG_CFG_D0	GENMASK(15,  8)
#define RFI_CTRLSIG_CFG_D1	GENMASK(15,  8)

#define TXEN			0x0
#define TXVALID			0x1
#define RXEN			0x2
#define ALWAYS			0x3

#define HIGH			0x0
#define LOW			0x1

#define RFCTRL(en, pol, on, off, d0, d1)	  \\
	(((en) << 21) |				  \\
	 ((pol) << 20) |			  \\
	 (((on) & 0x3) << 18) |			  \\
	 (((off) & 0x3) << 16) |		  \\
	 (((d0) & 0xff) << 8) |			  \\
	 ((d1) & 0xff))

RFI

cat << FOOTER

#endif /* __SCM80211_REGS_H__ */
FOOTER
