#!/usr/bin/env bash
# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2024 by KylinSoft
#
# Tool to make kernel self-extracting installation script

MAKESELF=$(which makeself 2>/dev/null)

if [ -z "${MAKESELF}" ]; then
    [ -f "$(which apt 2>/dev/null)" ] && sudo apt install makeself
    [ -f "$(which yum 2>/dev/null)" ] && sudo yum install makeself
fi

SCRIPTNAME=$(basename ${0})
ARCH=${ARCH:-$(uname -m)}
ARCH2=
TMPDIR=$(mktemp -d -p .)
KVER=
TARGET=
NO_HEADERS=
NO_VMLINUX=
NO_MODULES=

cleanup() {
    rm -rf ${TMPDIR}
}

trap cleanup EXIT

_help() {
    echo "Usage: ${SCRIPTNAME}"
	echo
	echo "Build kernel auto install script."
	echo
	echo "Options: "
	echo "	-s:           build single image archive."
	echo "	--no-vmlinux: skip vmlinux archive."
	echo "	--no-modules: skip modules archive."
	echo "	--no-headers: skip headers archive."
	echo "	--help, -h:   show this help."
	echo
    exit ${1}
}

OPTS="$(getopt -o 'sh' -l 'no-modules,no-vmlinux,no-headers,help' -n "${SCRIPTNAME}" -- "$@")" || exit 1
eval set -- "${OPTS}"
while true; do
    case "${1}" in
        -s)
            NO_VMLINUX=1
            NO_MODULES=1
            NO_HEADERS=1
            echo "archive single image"
            shift;;
        --no-vmlinux)
            NO_VMLINUX=1
            echo "no vmlinux"
            shift;;
        --no-headers)
            NO_HEADERS=1
            echo "no headers"
            shift;;
        --no-modules)
            NO_MODULES=1
            echo "no modules"
            shift;;
        --)
            shift
            break;;
        *)
            _help 1;;
    esac
done

case ${ARCH} in
    arm64|aarch64)
        ARCH=arm64
        ARCH2=aarch64
        TARGET=Image
        ;;
    mips|mips64el)
        ARCH=mips
        ARCH2=mips64el
        TARGET=vmlinuz
        ;;
    x86|x86_64)
        ARCH=x86
        ARCH2=x86_64
        TARGET=bzImage
        ;;
    *)
        exit 1
        ;;
esac

check_compiled() {
    if [ ! -f arch/${ARCH}/boot/${TARGET} ] ; then
        echo "kernel image not found!"
        exit 1
    fi
    if [ ! -n "${NO_MODULES}" -a ! -f modules.order ]; then
        echo "kernel modules not found!"
        echo "Try to compile modules or use --no-modules to avoid this error."
        exit 1
    fi
    KVER=$(make kernelrelease)
}

archive_modules()
{
    make INSTALL_MOD_PATH=${TMPDIR}/ INSTALL_MOD_STRIP=1 modules_install 1>/dev/null
    [ $? -gt 0 ] && exit 1
    tar -C ${TMPDIR}/lib/modules -cf ${TMPDIR}/modules-${KVER}.tar ${KVER}
    [ $? -gt 0 ] && exit 2
    rm -rf ${TMPDIR}/lib
}

archive_headers()
{
    local TMPFILE=$(mktemp -p ${TMPDIR})

    echo ".config" > ${TMPFILE}
    find * -path "debian" -prune -o -path "arch" -prune \
        -o -name "Kconfig*" -o -name "Makefile*" \
        -o -name "Module.*" >> ${TMPFILE}
    find arch/${ARCH} -name "Kconfig*" -o -name "Makefile*" -o -name "*.S" -o -name "*.h" >>  ${TMPFILE}
    find include/ -name "*" >> ${TMPFILE}
    find scripts/ -name "*" >> ${TMPFILE}
    sed -i s/^arch$//g ${TMPFILE}
    sed -i s/^debian$//g ${TMPFILE}
    tar -T ${TMPFILE} -cpf ${TMPDIR}/headers-${KVER}.tar
    rm -rf ${TMPFILE}
}

# kernel image
archive_image() {
    cp arch/${ARCH}/boot/${TARGET} ${TMPDIR}/
}

archive_vmlinux() {
    cp vmlinux ${TMPDIR}
}

check_compiled || exit 1

echo "archiving image"
archive_image

if [ -z "${NO_VMLINUX}" ]; then
    echo "archiving vmlinux"
    archive_vmlinux
fi

if [ -z "${NO_MODULES}" ]; then
    echo "archiving modules"
    archive_modules
fi

if [ -z "${NO_HEADERS}" ]; then
    echo "archiving headers"
    archive_headers
fi

# makeself
cat > ${TMPDIR}/install.sh << 'EOF'
#!/bin/bash

if [ x"ARCH2" != x"$(uname -m)" ]; then
    echo "This script only support ARCH2!" && exit 1
fi

DPKG_BIN=$(which dpkg 2>/dev/null)
RPM_BIN=$(which rpm 2>/dev/null)

install_debian() {
    cp TARGET /boot/vmlinuz-KVER \
        && chown root:root /boot/vmlinuz-KVER

    tar -C /lib/modules/ -xf modules-KVER.tar \
        && update-initramfs -c -k KVER

    if [ -f "headers-KVER.tar" ]; then
        mkdir /usr/src/linux-headers-KVER 2>/dev/null \
            || rm -rf /usr/src/linux-headers-KVER/*
        tar -C /usr/src/linux-headers-KVER -xf headers-KVER.tar \
            && chmod 755 /usr/src/linux-headers-KVER \
            && chown root:root -R /usr/src/linux-headers-KVER \
            && ln -sf /usr/src/linux-headers-KVER /lib/modules/KVER/build
    fi

    [ -f vmlinux ] && cp vmlinux /lib/modules/KVER/build

    #sudo ../grubby.x86_64 --grub2 --copy-default -c /boot/grub/grub.cfg --title="Test Kernel(5.4.18-25+)" \
    #   --add-kernel=/boot/bzImage-5.4.18-25+ --initrd /boot/initrd.img-5.4.18-25+ -o -
}

install_rhel() {
    cp TARGET /boot/vmlinuz-KVER \
        && chown root:root /boot/vmlinuz-KVER

    tar -C /lib/modules/ -xf modules-KVER.tar \
        && mkinitrd -f /boot/initramfs-KVER.img KVER

    if [ -f "headers-KVER.tar" ]; then
        mkdir /usr/src/kernels/KVER 2>/dev/null \
            || rm -rf /usr/src/kernels/KVER/*

        tar -C /usr/src/kernels/KVER -xf headers-KVER.tar \
            && chmod 755 /usr/src/kernels/KVER \
            && chown root:root -R /usr/src/kernels/KVER \
            && ln -sf /usr/src/kernels/KVER /lib/modules/KVER/build
    fi
    [ -f vmlinux ] && cp vmlinux /lib/modules/KVER/build
}

if [ -x "$DPKG_BIN" ] && [ $($DPKG_BIN -l | wc -l) -gt 0 ]; then
    install_debian
elif [ -x "$RPM_BIN" ] && [ $($RPM_BIN -qa | wc -l) -gt 0 ]; then
    install_rhel
else
    echo "Unsupport platform!"
    exit 1
fi
EOF

sed -i "s/ARCH2/${ARCH2}/g" ${TMPDIR}/install.sh
sed -i "s/KVER/${KVER}/g" ${TMPDIR}/install.sh
sed -i "s/TARGET/${TARGET}/g" ${TMPDIR}/install.sh
chmod a+x ${TMPDIR}/install.sh

echo "executing makeself ..."
makeself --needroot --bzip2 ${TMPDIR} kernel-${KVER}_${ARCH}.sh "kernel ${KVER} for ${ARCH}" ./install.sh
