#!/bin/bash

##############################################################
## 本脚本用于对代码做 kunit 单元测试
## 首先确保内核设置以下编译配置选项
##
## CONFIG_KUNIT=y
## CONFIG_KUNIT_DEBUGFS=y
## CONFIG_DEBUG_KERNEL=y
## CONFIG_DEBUG_INFO=y
## CONFIG_DEBUG_INFO_DWARF4=y
## CONFIG_GCOV_KERNEL=y
##
## 这个一定要关闭，否则会统计整个内核
## CONFIG_GCOV_PROFILE_ALL=n
##
##
## 然后安装 lcov工具覆盖率统计工具，最后确保ksaf-main 模块没有加载，然后直接执行此脚本即可
## 开启自动化测试
##
#############################################################


export SRC=$(dirname "$(realpath "$0")")
export WORK_DIR="/ksaf_main_kunit"

cd $SRC
source "$SRC"/kunit/lib.sh

check_env() {
        local commands=("lcov genhtml")
        for command in $commands; do
                if ! command -v $command &> /dev/null; then
                        print_fail "You need to install $command first!"
                        return 1
                fi
        done

        if [ ! -f "/sys/kernel/debug/gcov/reset" ]; then
                print_fail "Your kernel installed not support gcov!"
                return 1
        fi

        print_info "environment check success!"
}

build_modules() {
        print_info "buildding...."
        make -j$(nproc) KUNIT_TEST=1
}

gen_coverage_html() {
        lcov -c -b $SRC -d  /sys/kernel/debug/gcov/$SRC -o "$SRC"/result.info --exclude *.h --ignore-errors negative
        genhtml -o $SRC --ignore-errors inconsistent "$SRC"/result.info
}

processing()
{
        if [ $# -eq 2 ]; then
                timeout --foreground $2 bash -c '
                        while true;
                        do
                                string="\|/-"
                                for ((i = 0; i < ${#string}; i++))
                                do
                                        printf "%-s %-s \r" '"$1"' "${string:$i:1}"
                                        sleep 0.3
                                done
                        done
                '
	elif [ $# -eq 1 ];then
                while true;
                do
                        string="\|/-"
                        for ((i = 0; i < ${#string}; i++))
                        do
                                printf "%-s %-s \r" '"$1"' "${string:$i:1}"
                                sleep 0.3
                        done
                done
	fi
}

exit_work() {
        print_info "Remove directory $WORK_DIR"
        rm -rf $WORK_DIR;

        print_info "Killing backgroud jobs"
        [ -n "$(jobs -p)" ] && kill -9 $(jobs -p)
}

if ! check_env; then
        exit 1;
fi

trap 'exit_work' INT QUIT TERM EXIT

if lsmod | grep -q "^ksaf_main "; then
        print_info "ksaf main modules has been loaded, skip build!"
else
        set -e
        build_modules
        insmod "$SRC"/ksaf_main.ko ksaf_status=2
        set +e
fi

# 模块加载的比较晚，因此先初始化必要文件夹的标签
ksaf-label-manager-tool -a -s /usr/bin/

source "$SRC"/kunit/ksaf_test.sh
ksaf_kunit_test

# 等待一段时间，操作系统运行过程中，也会自动触发一些函数， 清理过期策略，至少需要60秒
print_info "Test finished!"
processing "Waiting... " 100s

# 测试完毕，生成代码覆盖率数据
gen_coverage_html
