#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/test_kysec_netlink/test_kysec_netlink.sh
 # @Description: perform kysec functional testing through the netlink interface
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###


# 获取当前内核版本并去掉后缀
current_version=$(uname -r | sed 's/-[^0-9]*$//')

###
 # @function: check_kernel_version
 # @description: compare the current kernel version with the target kernel version
 # @return {*}  0 - 当前内核版本($current_version)大于或者等于目标版本($target_version)
 #              -1 - 当前内核版本($current_version)小于目标版本($target_version)
###
function check_kernel_version {
    # 指定的版本并去掉后缀（如果需要的话
    target_version=$1
    target_version=$(echo "$target_version" | sed 's/-[^0-9]*$//')

    # 提取主要、次要和修订版本号
    current_major=${current_version%%.*}
    current_minor=${current_version#*.}
    current_minor=${current_minor%%.*}
    current_tmp=${current_version#*.*.*}
    current_revision=${current_tmp%%-*}
    current_release=${current_tmp#*-}
    current_release=${current_release%%.*}

    target_major=${target_version%%.*}
    target_minor=${target_version#*.}
    target_minor=${target_minor%%.*}
    target_tmp=${target_version#*.*.*}
    target_revision=${target_tmp%%-*}
    target_release=${target_tmp#*-}
    target_release=${target_release%%.*}

    # 进行数字比较
    if (( current_major > target_major )); then
        ret=0
    elif (( current_major == target_major && current_minor > target_minor )); then
        ret=0
    elif (( current_major == target_major && current_minor == target_minor && \
	    current_revision > target_revision )); then
        ret=0
    elif (( current_major == target_major && current_minor == target_minor && \
	    current_revision == target_revision && current_release > target_release )); then
        ret=0
    elif (( current_major == target_major && current_minor == target_minor && \
	    current_revision == target_revision && current_release == target_release )); then
        ret=0
        #echo "当前内核版本($current_version)等于目标版本($target_version)"
    else
        ret=-1
        #echo "当前内核版本($current_version)小于目标版本($target_version)"
    fi

    return $ret
}

TEST_PID_PROGRAM_PATH="./test_pid"
PID=0

TEST_KYSEC_NETLINK_PROGRAM="./test_kysec_netlink"

# 在后台执行程序
$TEST_PID_PROGRAM_PATH &

# 获取该程序的PID
PID=$!

###
 # @function: test_kysec_netctl_init
 # @description: kysec netctl init test
 # @return {*} 0: success; !0: failed
###
function test_kysec_netctl_init {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 0
}

###
 # @function: test_kysec_netctl_add
 # @description: kysec netctl add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_netctl_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 1
}

###
 # @function: test_kysec_netctl_del
 # @description: kysec netctl del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_netctl_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 2
}

###
 # @function: test_kysec_notify
 # @description: kysec notify test
 # @return {*} 0: success; !0: failed
###
function test_kysec_notify {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 3
}

###
 # @function: test_kysec_label_init
 # @description: kysec label init test
 # @return {*} 0: success; !0: failed
###
function test_kysec_label_init {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 4
}

###
 # @function: test_kysec_label_add
 # @description: kysec label add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_label_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 5
}

###
 # @function: test_kysec_label_del
 # @description: kysec label del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_label_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 6
}

###
 # @function: test_kysec_kmod_add
 # @description: kysec kmod add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_kmod_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 7
}

###
 # @function: test_kysec_kmod_del
 # @description: kysec kmod del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_kmod_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 8
}

###
 # @function: test_kysec_ppro_add
 # @description: kysec ppro add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ppro_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 9
}

###
 # @function: test_kysec_ppro_del
 # @description: kysec ppro del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ppro_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 10
}

###
 # @function: test_kysec_ppro_critical_pid
 # @description: kysec ppro critical pid test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ppro_critical_pid {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 11 $PID
}

###
 # @function: test_kysec_ppro_pid
 # @description: kysec ppro pid test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ppro_pid {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 12 $PID
}

###
 # @function: test_kysec_devctl_add
 # @description: kysec devctl add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_devctl_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 13
}

###
 # @function: test_kysec_devctl_del
 # @description: kysec devctl del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_devctl_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 14
}

###
 # @function: test_kysec_ssid_add
 # @description: kysec ssid add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ssid_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 15
}

###
 # @function: test_kysec_ssid_del
 # @description: kysec ssid del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ssid_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 16
}

###
 # @function: test_kysec_devctl_add_pro
 # @description: kysec devctl add pro test
 # @return {*} 0: success; !0: failed
###
function test_kysec_devctl_add_pro {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 17
}

###
 # @function: test_kysec_devctl_del_pro
 # @description: kysec devctl del pro test
 # @return {*} 0: success; !0: failed
###
function test_kysec_devctl_del_pro {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 18
}

###
 # @function: test_kysec_usb_port_sync
 # @description: kysec usb port sync test
 # @return {*} 0: success; !0: failed
###
function test_kysec_usb_port_sync {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 19
}

###
 # @function: test_kysec_pblk_add
 # @description: kysec pblk add test
 # @return {*} 0: success; !0: failed
###
function test_kysec_pblk_add {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 20
}

###
 # @function: test_kysec_pblk_del
 # @description: kysec pblk del test
 # @return {*} 0: success; !0: failed
###
function test_kysec_pblk_del {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 21
}

###
 # @function: test_kysec_kid_auth_clear
 # @description: kysec kid auth clear test
 # @return {*} 0: success; !0: failed
###
function test_kysec_kid_auth_clear {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi
    $TEST_KYSEC_NETLINK_PROGRAM 22 $PID
}

test_kysec_netctl_init
test_kysec_netctl_add
test_kysec_netctl_del
test_kysec_notify
test_kysec_label_init
test_kysec_label_add
test_kysec_label_del
test_kysec_kmod_add
test_kysec_kmod_del
test_kysec_ppro_add
test_kysec_ppro_del
test_kysec_ppro_critical_pid
test_kysec_ppro_pid
test_kysec_devctl_add
test_kysec_devctl_del
test_kysec_ssid_add
test_kysec_ssid_del
test_kysec_devctl_add_pro
test_kysec_devctl_del_pro
test_kysec_usb_port_sync
test_kysec_pblk_add
test_kysec_pblk_del
test_kysec_kid_auth_clear
