#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/test_kysec_sys/test_kysec_sys.sh
 # @Description: perform kysec functional testing through the sysfs interface
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###


# 获取当前内核版本并去掉后缀
current_version=$(uname -r | sed 's/-[^0-9]*$//')

###
 # @function: check_kernel_version
 # @description: compare the current kernel version with the target kernel version
 # @return {*}  0 - 当前内核版本($current_version)大于或者等于目标版本($target_version)
 #              -1 - 当前内核版本($current_version)小于目标版本($target_version)
###
function check_kernel_version {
    # 指定的版本并去掉后缀（如果需要的话
    target_version=$1
    target_version=$(echo "$target_version" | sed 's/-[^0-9]*$//')

    # 提取主要、次要和修订版本号
    current_major=${current_version%%.*}
    current_minor=${current_version#*.}
    current_minor=${current_minor%%.*}
    current_tmp=${current_version#*.*.*}
    current_revision=${current_tmp%%-*}
    current_release=${current_tmp#*-}
    current_release=${current_release%%.*}

    target_major=${target_version%%.*}
    target_minor=${target_version#*.}
    target_minor=${target_minor%%.*}
    target_tmp=${target_version#*.*.*}
    target_revision=${target_tmp%%-*}
    target_release=${target_tmp#*-}
    target_release=${target_release%%.*}


    # 进行数字比较
    if (( current_major > target_major )); then
        ret=0
    elif (( current_major == target_major && current_minor > target_minor )); then
        ret=0
    elif (( current_major == target_major && current_minor == target_minor && \
	    current_revision > target_revision )); then
        ret=0
    elif (( current_major == target_major && current_minor == target_minor && \
	    current_revision == target_revision && current_release > target_release )); then
        ret=0
    elif (( current_major == target_major && current_minor == target_minor && \
	    current_revision == target_revision && current_release == target_release )); then
        ret=0
        #echo "当前内核版本($current_version)等于目标版本($target_version)"
    else
        ret=-1
        #echo "当前内核版本($current_version)小于目标版本($target_version)"
    fi

    return $ret
}

###
 # @function: test_write_file
 # @description: write file test
 # @return {*} 0: success; !0: failed
###
function test_write_file {
    local FILE_PATH=$1
    local value=$2

    echo "$value" > "$FILE_PATH"
    if [ $? -ne 0 ]; then
        return -1
    fi
    return 0
}

###
 # @function: test_read_file
 # @description: read file test
 # @return {*} 0: success; !0: failed
###
function test_read_file {
    local FILE_PATH=$1
    local value=$2

    read_value=$(cat "$FILE_PATH")
    ret_read=$?

    if [ "$read_value" = "$value" ] && [ $ret_read -eq 0 ]; then
#       echo "状态值验证成功，读取到的值为$read_value"
        ret=0
    else
#        echo "状态值验证失败，读取到的值为$read_value"
        ret=-1
    fi
    return $ret
}

###
 # @function: test_file_interactive
 # @description: test read file and write file
 # @return {*} 0: success; !0: failed
###
function test_file_interactive {
    local FILE_PATH=$1
    local write_value=$2
    local read_value=$3
    local flag=$4

    test_write_file $FILE_PATH $write_value
    if [ $? -ne 0 ] && [ $flag -eq 0 ]; then
        return -1
    fi

    test_read_file $FILE_PATH $read_value
    if [ $? -ne 0 ]; then
        ret=-1
    fi

    return $ret
}

###
 # @function: test_kysec_status
 # @description: kysec status test
 # @return {*} 0: success; !0: failed
###
function test_kysec_status {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/status"
    test_file_interactive $FILE_PATH 4 4 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 2 0
    ret_3=$?

    test_file_interactive $FILE_PATH 5 4 1
    ret_4=$?

    test_file_interactive $FILE_PATH 0 0 0
    ret_5=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ] && [ $ret_4 -eq 0 ] && [ $ret_5 -eq 0 ]
    then
        echo "kysec status test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_status:res ok >> res_kysec_sys
    else
        echo "kysec status test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_status:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_version
 # @description: kysec version test
 # @return {*} 0: success; !0: failed
###
function test_kysec_version {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/version"

    expected_4_2="4.2"
    expected_4_3="4.3"
    expected_5_0="5.0"
    expected_5_1="5.1"
    expected_5_2="5.2"

    read_value=$(cat "$FILE_PATH")

    if [[ "$read_value" == "$expected_5_2" ]] || [[ "$read_value" == "$expected_5_1" ]] || \
            [[ "$read_value" == "$expected_5_0" ]] || [[ "$read_value" == "$expected_4_3" ]] || \
            [[ "$read_value" == "$expected_4_2" ]]
    then
        ret=0
    else
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_version:res failed >> res_kysec_sys
        echo "kysec version test failed"
        return -1
    fi

    echo "kysec version test ok, 读取到的kysec版本号为$read_value"
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_version:res ok >> res_kysec_sys
    return $ret
}

###
 # @function: test_kysec_fpro
 # @description: kysec fpro test
 # @return {*} 0: success; !0: failed
###
function test_kysec_fpro {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/fpro"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 1 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec fpro test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_fpro:res ok >> res_kysec_sys
    else
        echo "kysec fpro test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_fpro:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_exectl
 # @description: kysec exectl test
 # @return {*} 0: success; !0: failed
###
function test_kysec_exectl {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/exectl"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 3 2 1
    ret_4=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 2 0
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ] && [ $ret_4 -eq 0 ]
    then
        echo "kysec exectl test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_exectl:res ok >> res_kysec_sys
    else
        echo "kysec exectl test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_exectl:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_unnotify
 # @description: kysec unnotify test
 # @return {*} 0: success; !0: failed
###
function test_kysec_unnotify {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/unnotify"

    test_file_interactive $FILE_PATH "/opt/kingsoft/wps-office/office6/123456" \
	    "/opt/kingsoft/wps-office/office6/123456" 0

    if  [ $? -eq 0 ]
    then
        echo "kysec unnotify test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_unnotify:res ok >> res_kysec_sys
    else
        echo "kysec unnotify test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_unnotify:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_kmod
 # @description: kysec kmod test
 # @return {*} 0: success; !0: failed
###
function test_kysec_kmod {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/kmod"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 1 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec kmod test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kmod:res ok >> res_kysec_sys
    else
        echo "kysec kmod test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kmod:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_khtable
 # @description: kysec khtable test
 # @return {*} 0: success; !0: failed
###
function test_kysec_khtable {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/khtable"

    # 检查文件是否存在
    if [ ! -f "$FILE_PATH" ]; then
        echo "文件 $FILE_PATH 不存在"
        return -1
    fi

    # 定义期望的每一行的内容
    expected_total="total"
    expected_verified="verified"
    expected_original="original"
    expected_kysoft="kysoft"
    expected_trusted="trusted"
    expected_parent="parent"
    expected_script="script"

    while IFS= read -r line; do
        if [[ "$line" == "$expected_total"* ]] || [[ "$line" == "$expected_verified"* ]] || \
                [[ "$line" == "$expected_original"* ]] || [[ "$line" == "$expected_kysoft"* ]] || \
                [[ "$line" == "$expected_trusted"* ]] || [[ "$line" == "$expected_parent"* ]] || \
                [[ "$line" == "$expected_script"* ]]
        then
            ret=0
        else
            echo "kysec khtable test failed, 文件内容不符合预期格式: $line"
            echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_khtable:res failed >> res_kysec_sys
            return -1
        fi
    done < "$FILE_PATH"

    echo "kysec khtable test ok"
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_khtable:res ok >> res_kysec_sys
    return $ret
}

###
 # @function: test_kysec_netctl_info
 # @description: kysec netctl_info test
 # @return {*} 0: success; !0: failed
###
function test_kysec_netctl_info {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/netctl_info"

    # 检查文件是否存在
    if [ ! -f "$FILE_PATH" ]; then
        echo "文件 $FILE_PATH 不存在"
        return -1
    fi

    # 定义期望的每一行的内容
    expected_total="total"
    expected_allow="allow"
    expected_deny="deny"
    expected_auth="auth"

    # 读取文件的每一行并进行比较
    total_found=false
    allow_found=false
    deny_found=false
    auth_found=false

    while IFS= read -r line; do
        if [[ "$line" == "$expected_total"* ]]; then
            total_found=true
        elif [[ "$line" == "$expected_allow"* ]]; then
            allow_found=true
        elif [[ "$line" == "$expected_deny"* ]]; then
            deny_found=true
        elif [[ "$line" == "$expected_auth"* ]]; then
            auth_found=true
        else
            echo "kysec netctl_info test failed, 文件内容不符合预期格式: $line"
            echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_netctl_info:res failed >> res_kysec_sys
            return -1
        fi
    done < "$FILE_PATH"

    echo "kysec netctl_info test ok"
    ret=0
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_netctl_info:res ok >> res_kysec_sys

    return $ret
}

###
 # @function: test_kysec_netctl
 # @description: kysec netctl test
 # @return {*} 0: success; !0: failed
###
function test_kysec_netctl {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/netctl"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 2 0
    ret_3=$?

    test_file_interactive $FILE_PATH 3 3 1
    ret_4=$?

    test_file_interactive $FILE_PATH 4 0 1
    ret_5=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ] && [ $ret_4 -eq 0 ] && [ $ret_5 -eq 0 ]
    then
        echo "kysec netctl test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_netctl:res ok >> res_kysec_sys
    else
        echo "kysec netctl test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_netctl:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_netctl_user
 # @description: kysec netctl_user test
 # @return {*} 0: success; !0: failed
###
function test_kysec_netctl_user {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/netctl_user"
    test_file_interactive $FILE_PATH 1 1 0
    ret_1=$?

    test_file_interactive $FILE_PATH 0 0 0
    ret_2=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ]
    then
        echo "kysec netctl_user test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_netctl_user:res ok >> res_kysec_sys
    else
        echo "kysec netctl_user test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_netctl_user:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_ppro_info
 # @description: kysec ppro_info test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ppro_info {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/ppro_info"

    # 检查文件是否存在
    if [ ! -f "$FILE_PATH" ]; then
        echo "文件 $FILE_PATH 不存在"
        return -1
    fi

    # 定义期望的每一行的内容
    expected_total="total"
    expected_subs="SUBS"
    expected_key="KEY"

    # 读取文件的每一行并进行比较
    total_found=false
    subs_found=false
    key_found=false

    while IFS= read -r line; do
        if [[ "$line" == "$expected_total"* ]]; then
            total_found=true
        elif [[ "$line" == "$expected_subs"* ]]; then
            subs_found=true
        elif [[ "$line" == "$expected_key"* ]]; then
            key_found=true
        else
            echo "kysec ppro_info test failed, 文件内容不符合预期格式: $line"
            echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_ppro_info:res failed >> res_kysec_sys
            return -1
        fi
    done < "$FILE_PATH"

    echo "kysec ppro_info test ok"
    ret=0
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_ppro_info:res ok >> res_kysec_sys

    return $ret
}

###
 # @function: test_kysec_ppro
 # @description: kysec ppro test
 # @return {*} 0: success; !0: failed
###
function test_kysec_ppro {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/ppro"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec ppro test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_ppro:res ok >> res_kysec_sys
    else
        echo "kysec ppro test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_ppro:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_devctl_info
 # @description: kysec devctl_info test
 # @return {*} 0: success; !0: failed
###
function test_kysec_devctl_info {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/devctl_info"

    # 检查文件是否存在
    if [ ! -f "$FILE_PATH" ]; then
        echo "文件 $FILE_PATH 不存在"
        return -1
    fi

    read_value=$(cat "$FILE_PATH")
    ret_read=$?

    if [ $ret_read -eq 0 ]; then
        echo "kysec devctl_info test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_devctl_info:res ok >> res_kysec_sys
    else
        echo "kysec devctl_info test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_devctl_info:res failed >> res_kysec_sys
    fi
    return $ret
}

###
 # @function: test_kysec_devctl
 # @description: kysec devctl test
 # @return {*} 0: success; !0: failed
###
function test_kysec_devctl {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/devctl"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec devctl test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_devctl:res ok >> res_kysec_sys
    else
        echo "kysec devctl test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_devctl:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_device_info
 # @description: kysec device_info test
 # @return {*} 0: success; !0: failed
###
function test_kysec_device_info {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/device_info"

    # 检查文件是否存在
    if [ ! -f "$FILE_PATH" ]; then
        echo "文件 $FILE_PATH 不存在"
        return -1
    fi

    read_value=$(cat "$FILE_PATH")
    ret_read=$?

    if [ $ret_read -eq 0 ]; then
        echo "kysec device_info test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_device_info:res ok >> res_kysec_sys
    else
        echo "kysec device_info test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_device_info:res failed >> res_kysec_sys
    fi
    return $ret
}

###
 # @function: test_kysec_sm
 # @description: kysec sm test
 # @return {*} 0: success; !0: failed
###
function test_kysec_sm {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    devctl_status=$(cat "/sys/kernel/security/kysec/devctl")
    if [ $devctl_status -eq 0 ]; then
        echo 1 > "/sys/kernel/security/kysec/devctl"
    fi

    local FILE_PATH="/sys/kernel/security/kysec/sm"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec sm test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_sm:res ok >> res_kysec_sys
    else
        echo "kysec sm test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_sm:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_biosctl_info
 # @description: kysec biosctl_info test
 # @return {*} 0: success; !0: failed
###
function test_kysec_biosctl_info {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/biosctl_info"

    # 检查文件是否存在
    if [ ! -f "$FILE_PATH" ]; then
        echo "文件 $FILE_PATH 不存在"
        return -1
    fi

    read_value=$(cat "$FILE_PATH")
    ret_read=$?

    if [ $ret_read -eq 0 ]; then
        echo "kysec biosctl_info test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_biosctl_info:res ok >> res_kysec_sys
    else
        echo "kysec biosctl_info test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_biosctl_info:res failed >> res_kysec_sys
    fi
    return $ret
}

###
 # @function: test_kysec_pblk
 # @description: kysec pblk test
 # @return {*} 0: success; !0: failed
###
function test_kysec_pblk {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/pblk"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec pblk test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_pblk:res ok >> res_kysec_sys
    else
        echo "kysec pblk test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_pblk:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_pblk_info
 # @description: kysec pblk_info test
 # @return {*} 0: success; !0: failed
###
function test_kysec_pblk_info {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    check_kernel_version "5.4.18-97"
    if [ $? -eq 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/pblk_info"

    expected_total="total"

    read_value=$(cat "$FILE_PATH")

    if [[ "$read_value" == "$expected_5_2" ]]
    then
        ret=0
    else
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_pblk_info:res failed >> res_kysec_sys
        echo "kysec pblk_info test failed"
        return -1
    fi

    echo "kysec pblk_info test ok"
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_pblk_info:res ok >> res_kysec_sys
    return $ret
}

###
 # @function: test_kysec_iptables
 # @description: kysec iptables test
 # @return {*} 0: success; !0: failed
###
function test_kysec_iptables {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/iptables"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec iptables test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_iptables:res ok >> res_kysec_sys
    else
        echo "kysec iptables test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_iptables:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_eperm_status
 # @description: kysec eperm_status test
 # @return {*} 0: success; !0: failed
###
function test_kysec_eperm_status {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/eperm_status"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec eperm_status test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_eperm_status:res ok >> res_kysec_sys
    else
        echo "kysec eperm_status test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_eperm_status:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_kid
 # @description: kysec kid test
 # @return {*} 0: success; !0: failed
###
function test_kysec_kid {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/kid"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 2 0
    ret_3=$?

    test_file_interactive $FILE_PATH 3 3 0
    ret_4=$?

    test_file_interactive $FILE_PATH 4 3 1
    ret_5=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ] && [ $ret_4 -eq 0 ] && [ $ret_5 -eq 0 ]
    then
        echo "kysec kid test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kid:res ok >> res_kysec_sys
    else
        echo "kysec kid test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kid:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: kid_policy_test
 # @description: kysec kid policy test
 # @return {*} 0: success; !0: failed
###
function kid_policy_test {
    local ADD_POLICY_PATH=$1
    local REMOVE_POLICY_PATH=$2

    local FILE_PATH="/sys/kernel/security/kysec/kid_policy"
    policy_count_1=$(cat "$FILE_PATH")
    if [ $? -ne 0 ]; then
        echo "kysec kid policy test failed1......"
        return -1
    fi

    cat "$ADD_POLICY_PATH" > "$FILE_PATH"
    if [ $? -ne 0 ]; then
        echo "kysec kid policy test failed2......"
        return -1
    fi

    policy_count_2=$(cat "$FILE_PATH")
    if [ $? -ne 0 ]; then
        echo "kysec kid policy test failed3......"
        return -1
    fi

    cat "$REMOVE_POLICY_PATH" > "$FILE_PATH"
    if [ $? -ne 0 ]; then
        echo "kysec kid policy test failed4......"
        return -1
    fi

    policy_count_3=$(cat "$FILE_PATH")
    if [ $? -ne 0 ]; then
        echo "kysec kid policy test failed5......"
        return -1
    fi

    if (( policy_count_2 == policy_count_1 + 1 )); then
        if [ "$policy_count_1" -eq "$policy_count_3" ]; then
            echo "kysec kid policy test ok......"
            ret=0
        else
            echo "kysec kid policy test failed6......"
            ret=-1
        fi
    else
        echo "kysec kid policy test failed7......"
        ret=-1
    fi

    return $ret
}

###
 # @function: test_kysec_kid_policy
 # @description: kysec kid policy test
 # @return {*} 0: success; !0: failed
###
function test_kysec_kid_policy {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -eq 0 ]; then
        kid_policy_test "./kid_policy/kid_policy_new_add" "./kid_policy/kid_policy_new_remove"
        if [ $? -ne 0 ]; then
            echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kid_policy:res failed >> res_kysec_sys
            return -1
        fi
    fi

    kid_policy_test "./kid_policy/kid_policy_old_add" "./kid_policy/kid_policy_old_remove"
    ret=$?

    if  [ $ret -eq 0 ]; then
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kid_policy:res ok >> res_kysec_sys
    else
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_kid_policy:res failed >> res_kysec_sys
    fi

    return $ret
}

###
 # @function: test_kysec_nsctl
 # @description: kysec nsctl test
 # @return {*} 0: success; !0: failed
###
function test_kysec_nsctl {
    check_kernel_version "6.6.0-7.0.v2405"
    if [ $? -ne 0 ]; then
        echo "kernel version is not support"
        return 0
    fi

    local FILE_PATH="/sys/kernel/security/kysec/nsctl"
    test_file_interactive $FILE_PATH 0 0 0
    ret_1=$?

    test_file_interactive $FILE_PATH 1 1 0
    ret_2=$?

    test_file_interactive $FILE_PATH 2 0 1
    ret_3=$?

    if  [ $ret_1 -eq 0 ] && [ $ret_2 -eq 0 ] && [ $ret_3 -eq 0 ]
    then
        echo "kysec nsctl test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_nsctl:res ok >> res_kysec_sys
    else
        echo "kysec nsctl test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:test_kysec_nsctl:res failed >> res_kysec_sys
    fi

    return $ret
}



test_kysec_kid_policy
res_test_kysec_kid_policy=$?
sleep 1

test_kysec_kid
res_test_kysec_kid=$?
sleep 1

test_kysec_version
res_test_kysec_version=$?
sleep 1

test_kysec_fpro
res_test_kysec_fpro=$?
sleep 1

test_kysec_exectl
res_ttest_kysec_exectl=$?
sleep 1

test_kysec_unnotify
res_test_kysec_unnotify=$?
sleep 1

test_kysec_kmod
res_test_kysec_kmod=$?
sleep 1

test_kysec_khtable
res_test_kysec_khtable=$?
sleep 1

test_kysec_netctl_info
res_test_kysec_netctl_info=$?
sleep 1

test_kysec_netctl_user
res_test_kysec_netctl_user=$?
sleep 1

test_kysec_netctl
res_test_kysec_netctl=$?
sleep 1

test_kysec_ppro_info
res_test_kysec_ppro_info=$?
sleep 1

test_kysec_ppro
res_test_kysec_ppro=$?
sleep 1

test_kysec_devctl_info
res_test_kysec_devctl_info=$?
sleep 1

test_kysec_device_info
res_test_kysec_device_info=$?
sleep 1

test_kysec_devctl
res_test_kysec_devctl=$?
sleep 1

test_kysec_biosctl_info
res_test_kysec_biosctl_info=$?
sleep 1

test_kysec_sm
res_test_kysec_sm=$?
sleep 1

test_kysec_pblk
res_test_kysec_pblk=$?
sleep 1

test_kysec_pblk_info
res_test_kysec_pblk_info=$?
sleep 1

test_kysec_iptables
res_test_kysec_iptables=$?
sleep 1

test_kysec_eperm_status
res_test_kysec_eperm_status=$?
sleep 1

test_kysec_status
res_test_kysec_status=$?
sleep 1

test_kysec_nsctl
res_test_kysec_nsctl=$?
sleep 1

if [ $res_test_kysec_status -eq 0 ] && [ $res_test_kysec_version -eq 0 ] && [ $res_test_kysec_fpro -eq 0 ] \
	&& [ $res_ttest_kysec_exectl -eq 0 ] && [ $res_test_kysec_unnotify -eq 0 ] \
        && [ $res_test_kysec_kmod -eq 0 ] && [ $res_test_kysec_khtable -eq 0 ] \
	&& [ $res_test_kysec_netctl_info -eq 0 ] && [ $res_test_kysec_netctl -eq 0 ] \
        && [ $res_test_kysec_netctl_user -eq 0 ] && [ $res_test_kysec_ppro_info -eq 0 ] \
	&& [ $res_test_kysec_ppro -eq 0 ] && [ $res_test_kysec_devctl_info -eq 0 ] \
        && [ $res_test_kysec_devctl -eq 0 ] && [ $res_test_kysec_device_info -eq 0 ] \
	&& [ $res_test_kysec_sm -eq 0 ] && [ $res_test_kysec_biosctl_info -eq 0 ] \
        && [ $res_test_kysec_pblk -eq 0 ] && [ $res_test_kysec_iptables -eq 0 ] \
	&& [ $res_test_kysec_eperm_status -eq 0 ] && [ $res_test_kysec_kid -eq 0 ] \
        && [ $res_test_kysec_kid_policy -eq 0 ] && [ $res_test_kysec_nsctl -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:testcase:res ok >> res_kysec_sys
else
    echo `date +%Y%m%d%H%M%S`:test_kysec_interactive:testcase:res failed >> res_kysec_sys
fi
