#!/bin/bash
# SPDX-License-Identifier: GPL-2.0

# File Name: test_mem_ratio_pressure.sh
# Author: liubaolin
# mail: liubaolin@kylinos.cn

# Tool background:
# Some tests require testing under 90% memory usage conditions, such as LTP.
# Usually, memtester is used to occupy memory.
# However, due to the OOM mechanism in the kernel,
# it is possible to kill the memtester program in the event of insufficient memory.
# Although it is possible to force the oom not to be killed by
# modifying the oom score of the memtester process.
# But this can cause other desktop processes such as Xorg to be killed by OOM.
# So in order to maintain the memory usage of 90% during testing.
# You need to check regularly whether the memtester process has been killed,
# If killed, restart the memtester process according to the memory usage
# requirements of the test
#
# Usage:
# First, place the testMemtesterRatio.sh script in/root
# vim /etc/crontab
# Add a row
#	* * * * * root cd /root && ./testMemtesterRatio.sh
#	This is checked every minute.
#	1 * * * * root cd /root && ./testMemtesterRatio.sh
# This is tested every hour.
#
# Then the testMemtesterRatio.sh script takes two parameters,
# The first parameter $1 is the memory usage ratio.
# For example, if 90% of the memory needs to be used for testing,
# then the first parameter is set to 0.9.
# The second parameter $2 is the directory where the memster
# executable program is located
#
# Be sure to run ltp first, and then add this scheduled task.
#
# The testMemtesterRatio.sh script is scheduled to check
# if the memtester process has been killed,
# If killed, calculate how much more memory is needed to occupy 90% of the memory
# Then re execute memtester
# Note:
# This testing tool can be modified to use scenarios similar to
# "occupying fixed memory for related testing"

stressradio=$1
memtesterDir=$2
cd $2
echo come in my shell22 >> /root/testMem.log

# Check if the memtester process is still running and if it has been killed by OOM
process=`ps -fe | grep memtester | grep -v grep | wc -l`

if [ $process -eq 0 ];then
	# If the memtester process has been killed,
	# restart the memster according to the testing memory requirements
	echo $(date +%Y-%m-%d\ %H:%M:%S) >> /root/testMem.log
	echo liubaolin rerun >> /root/testMem.log

	# View the total memory size of the system
	totalMem=`free | awk '/Mem/ {printf $2}'`

	# View the system's used memory
	useMem=`free | awk '/Mem/ {printf $3"\n"}'`

	# Memory required for calculating pressure
	stressMem=`echo $totalMem $stressradio | awk '{printf "%d\n", $1*$2}'`

	# Subtract the used memory from the memory required for
	# pressure to obtain the size of memory
	# that needs to be pressurized by the memster
	stressDeleteUse=`echo $stressMem $useMem | awk '{printf "%d\n", $1-$2}'`

	# Convert memory units to G
	kbToG=1048576
	stressDeleteUseG=`echo $stressDeleteUse $kbToG | awk '{printf "%d\n", $1/$2}'`

	if [ $stressDeleteUseG -gt 0 ]
	then
		memtesterCmd="./memtester "$stressDeleteUseG"G"
		# Restart memtester according to the memory requirements for testing pressure
		memtesterExec=`$memtesterCmd`
	else
		echo mem is over radio,may not run memtester >> /root/testMem.log
	fi

else
	echo $(date +%Y-%m-%d\ %H:%M:%S) >> /root/testMem.log
	echo memtester is alive >> /root/testMem.log
fi
